//////////////////////////////////////////////////////////////////////////////
// Module   : sx16.h                                                        //
// Product  : Sensoray 516/616 driver.                                      //
// Function : The sx16_ioctl() commands.                                    //
//                                                                          //
// Examples:                                                                //
//    sx16_ioctl( handle, SX16_IOC_SET_INPUT, 0 );                          //
//    sx16_ioctl( handle, SX16_IOC_RESET );                                 //
//    br = sx16_ioctl( handle, SX16_IOC_GET_SETTING, ATTR_video_bitrate );  //
//////////////////////////////////////////////////////////////////////////////

#ifndef _SX16IOCTL_H
#define _SX16IOCTL_H

#ifdef WIN32
	#define IOC_OUT_DIRECT( id_lin, id_win )	CTL_CODE( FILE_DEVICE_UNKNOWN, id_win, METHOD_OUT_DIRECT, FILE_ANY_ACCESS ) 
	#define IOC_IN_DIRECT(  id_lin, id_win )	CTL_CODE( FILE_DEVICE_UNKNOWN, id_win, METHOD_IN_DIRECT , FILE_ANY_ACCESS ) 
	#define IOC_BUFFERED(   id_lin, id_win )	CTL_CODE( FILE_DEVICE_UNKNOWN, id_win, METHOD_BUFFERED,   FILE_ANY_ACCESS ) 
	#define IOC_NEITHER(    id_lin, id_win )	CTL_CODE( FILE_DEVICE_UNKNOWN, id_win, METHOD_NEITHER,    FILE_ANY_ACCESS ) 
#else    
	#define IOC_OUT_DIRECT( id_lin, id_win )	id_lin 
	#define IOC_IN_DIRECT(  id_lin, id_win )	id_lin 
	#define IOC_BUFFERED(   id_lin, id_win )	id_lin 
	#define IOC_NEITHER(    id_lin, id_win )	id_lin 
#endif

/*
NOTES:

1. These are the legal combinations for stream/mpeg modes, which are programmed via
   the SX16_IOC_SET_STREAM_TYPE and SX16_IOC_SET_MPEG ioctls:

	+------Operating mode-----+-------Settings-------+
	| Stream		Mpeg	  | STREAM_TYPE		MPEG |
	+-------------------------+----------------------+
	| Multiplexed	2:D1-704  | 	0			 0	 |
	| 	"			2:2/3 D1  | 	0			 1	 |
	| 	"			2:1/2 D1  | 	0			 2	 |
	| 	"			1:SIF	  | 	0			 3	 |
	| 	"			1:QSIF	  | 	0			 4	 |
	| 	"			2:D1-720  | 	0			 5	 |
	| 	"			2:D1-640  | 	0			 6	 |
	|	"			1:SIF	  | 	0			 7	 |
    |	"			2:2/3 D1  | 	0			 8	 |
	+-------------------------+----------------------+
	|  VES			2:D1-704  | 	1			 0	 |
	| 	"			2:2/3 D1  | 	1			 1	 |
	| 	"			2:1/2 D1  | 	1			 2	 |
	| 	"			1:SIF	  | 	1			 3	 |
	| 	"			1:QSIF	  | 	1			 4	 |
	| 	"			2:D1-720  | 	1			 5	 |
	| 	"			2:D1-640  | 	1			 6	 |
	+-------------------------+----------------------+
	+  AES			 n/a	  |		2		    n/a	 |
	+-------------------------+----------------------+
	| TRANSPORT		2:D1-704  |     3            0   | // encoding only
	+------------------------------------------------+
	Notes:
	a) Multiplexed modes 3 and 7 differ in multiplexing format: mode 3 has VCD, mode 7 - DVD multiplexing
	b) Playback of multiplexed stream is supported only for the following MPEG modes: 0, 7.
	c) Multiplexed modes 1 and 8 differ in multiplexing format: mode 1 has SVCD, mode 8 - DVD multiplexing
    d) Playback of TS is not currently supported
    
2. These are the legal combinations for i-p distance and group-of-pictures, which are programmed via the SX16_IOC_SET_N
   and SX16_IOC_SET_M ioctls; all other combinations will cause unpredictable behavior:

	N	M					GOP Format		Comments
	---------------------	--------------------------------------------------------
	1	1					iiiiiii...		i-frames only; works with NTSC and PAL.
	1	2,3,4,5, ... ,31	ipppp...		no b-frames; works with NTSC and PAL.
	2	4,6,8,10, ... ,30	ibpbpbp...		works with NTSC only.
	3	6,9,12,15, .. ,30	ibbpbbpbbp...	works with NTSC only.

*/
#define SX16_STREAMTYPE_MUX 0
#define SX16_STREAMTYPE_VES 1
#define SX16_STREAMTYPE_AES 2
#define SX16_STREAMTYPE_TS  3  // Encoding only.  Decoding of TS is currently not supported.  

//===============================================================================================================
// Get board type.

#define SX16_IOC_GET_BTYPE				IOC_OUT_DIRECT	( 0x55ff, 0x8f0 ) // Read board type: 516/616.

//===============================================================================================================
// Uncompressed capture control (616 only).
// currently not supported on Windows driver
#ifndef WIN32
#define SX16_IOC_CAPTURE_OPEN			IOC_IN_DIRECT 	( 0x0000, 0x800 ) // Prepare for capture.
#define SX16_IOC_CAPTURE_STATUS			IOC_IN_DIRECT 	( 0x0100, 0x802 ) // Get capture status: returns non-zero if capture is in progress.
#define SX16_IOC_CAPTURE_START			IOC_IN_DIRECT 	( 0x0200, 0x804 ) // Start capture.
#define SX16_IOC_CAPTURE_CLOSE			IOC_IN_DIRECT 	( 0x0300, 0x806 ) // Stop and close capture.
#endif

//===============================================================================================================
// General-purpose I/O port.  Bit is set to 1 when the associated I/O is driven low.

#define SX16_IOC_GET_GPIO				IOC_OUT_DIRECT	( 0x1000, 0x808 ) // Read state of GPIO pins.
#define SX16_IOC_SET_GPIO				IOC_IN_DIRECT 	( 0x1100, 0x80a ) // Set state of GPIO pins.

//===============================================================================================================
// Video and mpeg codec configuration.

#define SX16_IOC_GET_SETTING			IOC_OUT_DIRECT	( 0x2000, 0x80c ) // Return current configuration setting.
#define SX16_IOC_SET_AUDIO_BITRATE		IOC_IN_DIRECT	( 0x2100, 0x80e ) // Set audio bit rate.
// AUDIO_BITRATE settings 
// 1 - 32kbits/s
// 2 - 48kbits/s
// 3 - 56kbits/s
// 4 - 64kbits/s
// 5 - 80kbits/s
// 6 - 96kbits/s
// 7 - 112kbits/s
// 8 - 128kbits/s
// 9 - 160kbits/s
// 10 - 192kbits/s
// 11 - 224kbits/s(default recommended setting)
// 12 - 256kbits/s
// 13 - 320kbits/s
// 14 - 384kbits/s
//#define SX16_IOC_SET_AUDIO_SMPRATE   // see below
#define SX16_IOC_SET_AUDIO_FORMAT		IOC_IN_DIRECT 	( 0x2110, 0x810 ) // Set audio format: AFMT_MPEG1_LAYER1, ...
#define SX16_IOC_SET_AUDIO_MUTE			IOC_IN_DIRECT 	( 0x2120, 0x812 ) // Enable/disable (1/0) audio mute.
#define SX16_IOC_SET_AUDIO_STEREO		IOC_IN_DIRECT 	( 0x2130, 0x814 ) // Set stereo mode: STEREOMODE_STEREO, STEREOMODE_JOINT, STEREOMODE_DUAL or STEREOMODE_SINGLE.
#define SX16_IOC_SET_AUDIO_VOLUME		IOC_IN_DIRECT 	( 0x2140, 0x816 ) // Set audio volume (-dB). 0=maximum, 63=minimum.
#define SX16_IOC_SET_AV_SELECT			IOC_IN_DIRECT 	( 0x2150, 0x818 ) // Set a/v select.
#define SX16_IOC_SET_BITRATE			IOC_IN_DIRECT 	( 0x2160, 0x81a ) // Set desired bit rate, typically 500,000 to 5,000,000.
#define SX16_IOC_SET_BRIGHTNESS			IOC_IN_DIRECT 	( 0x2170, 0x81c ) // Set brightness for video in.
#define SX16_IOC_SET_COLORBARS			IOC_IN_DIRECT 	( 0x2180, 0x81e ) // Enable/disable (1/0) colors bars on video out.
#define SX16_IOC_SET_CONTRAST			IOC_IN_DIRECT 	( 0x2190, 0x820 ) // Set contrast for video in.
#define SX16_IOC_SET_FILTER				IOC_IN_DIRECT 	( 0x21a0, 0x822 ) // Set filter for low quality input.
#define SX16_IOC_SET_INPUT				IOC_IN_DIRECT 	( 0x21b0, 0x824 ) // Select video input, i.e. 0 - 3.
#define SX16_IOC_SET_INPUT_TYPE			IOC_IN_DIRECT 	( 0x21c0, 0x826 ) // Set video input type, i.e. SVIDEO (1) or CVIDEO (0).
#define SX16_IOC_SET_M					IOC_IN_DIRECT 	( 0x21d0, 0x828 ) // Set number of frames in Group of Pictures (1...15, default = 15).
#define SX16_IOC_SET_MPEG				IOC_IN_DIRECT 	( 0x21e0, 0x82a ) // Set picture size and compression type: see note 1 above.
#define SX16_IOC_SET_N					IOC_IN_DIRECT 	( 0x21f0, 0x82c ) // Set distance between I/P frames (1...3, default = 3).
#define SX16_IOC_SET_PROG_NUM			IOC_IN_DIRECT 	( 0x2200, 0x82e ) // Set program number if using transport stream.
#define SX16_IOC_SET_SATURATION			IOC_IN_DIRECT 	( 0x2210, 0x830 ) // Set saturation level for video in.
#define SX16_IOC_SET_STREAM_TYPE		IOC_IN_DIRECT 	( 0x2220, 0x832 ) // Set stream type: STREAMTYPE_MUX (0) | STREAMTYPE_VES (1) | STREAMTYPE_AES (2) | STREAMTYPE_TRANSPORT (3).  See note 1 above.
#define SX16_IOC_SET_VBR				IOC_IN_DIRECT 	( 0x2230, 0x834 ) // Enable/disable (1/0) variable bit rate compression.
#define SX16_IOC_SET_VIDEO_SYSTEM		IOC_IN_DIRECT 	( 0x2240, 0x836 ) // Set input & output video system, i.e. NTSC (2) or PAL (1).


//===============================================================================================================
// Stream access and control.

#define SX16_IOC_KILL					IOC_NEITHER   	( 0x3000, 0x838 ) // Kill blocking operation.


//===============================================================================================================
// Video output shut off control.
#define SX16_IOC_SET_ENCODER_OUT			IOC_IN_DIRECT	( 0x2242, 0x8a0 ) // Video output shut off control.


// 0x3002 to 0x0x31ff are RESERVED(see unsupported ioctls below)

#define SX16_IOC_RESET					IOC_NEITHER		( 0x3200, 0x840 ) // Reset the board.
#define SX16_IOC_RW_STOP				IOC_NEITHER		( 0x3300, 0x842 ) // Stop read/write.
#define SX16_IOC_RW_PAUSE				IOC_NEITHER		( 0x3400, 0x844 ) // Pause read/write.
#define SX16_IOC_RW_RESUME				IOC_NEITHER		( 0x3500, 0x846 ) // Resume read/write after pause.




//===============================================================================================================
// Video caption overlay.

#define SX16_IOC_GET_CAPTION_SETTINGS	IOC_OUT_DIRECT	( 0x4000, 0x85e ) // Get caption settings.
#define SX16_IOC_SET_CAPTION_WINDOW		IOC_IN_DIRECT	( 0x4100, 0x860 ) // Set caption window number.
#define SX16_IOC_SET_CAPTION_XSTART		IOC_IN_DIRECT	( 0x4200, 0x862 ) // Set caption window position: X_start.
#define SX16_IOC_SET_CAPTION_YSTART		IOC_IN_DIRECT	( 0x4300, 0x864 ) // Set caption window position: Y_start.
#define SX16_IOC_SET_CAPTION_SCALE		IOC_IN_DIRECT	( 0x4400, 0x866 ) // Set caption font scale;
#define SX16_IOC_SET_CAPTION_COLOR		IOC_IN_DIRECT	( 0x4500, 0x868 ) // Set caption color
#define SX16_IOC_SET_CAPTION_TRANSPARENCY IOC_IN_DIRECT	( 0x4600, 0x86a ) // Set caption transparency attribute
#define SX16_IOC_SET_CAPTION_TEXT		IOC_IN_DIRECT	( 0x4700, 0x86c ) // Set caption text to overlay
#define SX16_IOC_SET_CAPTION_ON			IOC_IN_DIRECT	( 0x4800, 0x86e ) // Set caption on/off

//===============================================================================================================
// NOT RECOMMENDED FOR NEW DESIGNS--these are depracated and/or intended only for Sensoray test and diagnostics.

#define SX16_IOC_GA_LOOPBACK			IOC_NEITHER		( 0x5000, 0x870 ) // Enable/disable video stream loopback (bypass mpeg codec).
#define SX16_IOC_GET_Q_SIZE		  		IOC_OUT_DIRECT	( 0x5100, 0x872 ) // Get the MPEG queue size.
// functions below have been removed
//#define SX16_IOC_SET_MUX_FORMAT			IOC_IN_DIRECT	( 0x5200, 0x874 ) // Set mpeg multiplex format.
//#define SX16_IOC_SET_PICSIZE			IOC_IN_DIRECT	( 0x5300, 0x876 ) // Set picture size and compression type.
//#define SX16_IOC_SET_SYS_FORMAT			IOC_IN_DIRECT	( 0x5400, 0x878 ) // Set system format.  ALWAYS SET THIS TO 2.

//=============================================================================================================

// 0x5500 is reserved( see unsupported functions below)

#define SX16_IOC_SET_AUDIO_SMPRATE		IOC_IN_DIRECT	( 0x5510, 0x87c ) // 616 only Set audio sample rate.  0-44.1(616 only) 1-48KHz 2-32(616 only)
#define SX16_IOC_GET_INPUT_STATUS		IOC_IN_DIRECT 	( 0x5520, 0x880 ) // Get status of video input.(616 and 516 Rev E+ only)
#define SX16_IOC_WAIT_VIDEO		        IOC_IN_DIRECT 	( 0x5530, 0x882 ) // Wait for video sync (616 only)

// CAPTURE settings 
// not supported yet
// Use video for Linux
//#define SX16_IOC_VIDCAP_QUERYCAP        IOC_OUT_DIRECT  ( 0x5530, 0x882 )
//#define SX16_IOC_VIDCAP_ENUMINPUT       IOC_OUT_DIRECT  ( 0x5540, 0x884 )
//#define SX16_IOC_VIDCAP_ENUMOUTPUT      IOC_OUT_DIRECT  ( 0x5550, 0x886 )
//#define SX16_IOC_VIDCAP_GETFMT          IOC_OUT_DIRECT  ( 0x5560, 0x888 )
//#define SX16_IOC_VIDCAP_SETFMT          IOC_IN_DIRECT   ( 0x5570, 0x88a )
//#define SX16_IOC_VIDCAP_REQBUFS         IOC_OUT_DIRECT  ( 0x5580, 0x88c )
//#define SX16_IOC_VIDCAP_QBUF            IOC_IN_DIRECT   ( 0x5590, 0x890 )
//#define SX16_IOC_VIDCAP_DQBUF           IOC_OUT_DIRECT  ( 0x55a0, 0x892 )
//#define SX16_IOC_VIDCAP_STREAMON        IOC_NEITHER     ( 0x55b0, 0x894 )
//#define SX16_IOC_VIDCAP_STREAMOFF       IOC_NEITHER     ( 0x55c0, 0x896 )
// 10/23/2007  ioctls for changing audio chip
// 516
#define SX16_IOC_SET_AUDIO_AGCON		IOC_IN_DIRECT	( 0x5700, 0x8b0 ) 
#define SX16_IOC_SET_AUDIO_AGCSENS		IOC_IN_DIRECT	( 0x5710, 0x8b2 ) 
#define SX16_IOC_SET_AUDIO_AMPGAIN		IOC_IN_DIRECT	( 0x5720, 0x8b4 ) 
#define SX16_IOC_SET_AUDIO_TREBLE		IOC_IN_DIRECT	( 0x5730, 0x8b6 ) 
#define SX16_IOC_SET_AUDIO_BASS 		IOC_IN_DIRECT	( 0x5740, 0x8b8 ) 


//===============================================================================================================
// MPEG queue identifiers use with SX16_IOC_GET_Q_SIZE 
#define Q_ID_AUD						1		// Audio queue.
#define Q_ID_MUX						2		// Multiplexed queue.
#define Q_ID_VID						3		// Video queue.

//===============================================================================================================
//the functions below are unsupported.  

// the following IOCTLs are reserved and built for 2416 product only. 
// Source code not available.
#define SX16_IOC_RD_SENSORAY			IOC_OUT_DIRECT	( 0x3100, 0x83a ) // Transfer encoded mpeg data (in Sensoray packetized format) to user buffer. 1440 bytes
#ifdef  SX16_BUILD_SRAY
#define SX16_IOC_WR_SENSORAY_NET		IOC_IN_DIRECT	( 0x3180, 0x83c ) // Transfer encoded mpeg data (in Sensoray packetized format). Discard data while board not ready
#define SX16_IOC_WR_SENSORAY_FILE		IOC_IN_DIRECT	( 0x3190, 0x83e ) // Transfer encoded mpeg data (in Sensoray packetized format). Block while board not ready.
#endif

#ifndef WIN32
#define SX16_IOC_DECODER_READY   		IOC_IN_DIRECT	( 0x55d0, 0x898 ) // Is decoder firmware uploaded?
#define SX16_IOC_ENCODER_READY   		IOC_IN_DIRECT	( 0x55e0, 0x89a ) // Is decoder firmware uploaded?
#define SX16_IOC_START_DECODER   		IOC_IN_DIRECT	( 0x55f0, 0x89c ) // upload decoder firmware   // val ==1 non-blocking
#define SX16_IOC_START_ENCODER   		IOC_IN_DIRECT	( 0x5600, 0x89f ) // upload encoder firmware
#endif

#define SX16_IOC_RD_SENSORAY_TS			IOC_OUT_DIRECT	( 0x5500, 0x87a ) // Transfer encoded mpeg data (in Sensoray packetized format) to user buffer. 1316 bytes transport stream




/* RTP commands are for the 2416 product only */

#ifndef WIN32
#define SX16_IOC_RD_RTP					IOC_OUT_DIRECT	( 0x3600, 0x848 ) // Transfer encoded mpeg data (in RTP format) to user buffer.
#define SX16_IOC_RTP_PLUS				IOC_OUT_DIRECT	( 0x3700, 0x84a ) // Request "raw" data along with packetized RTP
#endif

//===============================================================================================================
// RTCP reporting.
#ifndef WIN32
#define SX16_IOC_GET_RTCP_SR			IOC_OUT_DIRECT	( 0x3610, 0x84c )
#define SX16_IOC_GET_RTCP_RR			IOC_OUT_DIRECT	( 0x3620, 0x84e )
#define SX16_IOC_PUT_RTCP_RR			IOC_IN_DIRECT	( 0x3630, 0x850 )
#define SX16_IOC_PUT_RTCP_SR			IOC_IN_DIRECT	( 0x3640, 0x852 )
#define SX16_IOC_GET_SSCP_SR			IOC_OUT_DIRECT	( 0x3650, 0x854 )
#define SX16_IOC_GET_SSCP_RR			IOC_OUT_DIRECT	( 0x3660, 0x856 )
#define SX16_IOC_PUT_SSCP_RR			IOC_IN_DIRECT	( 0x3670, 0x858 )
#define SX16_IOC_PUT_SSCP_SR			IOC_IN_DIRECT	( 0x3680, 0x85a )
#define SX16_IOC_GET_RTCPA_SR			IOC_OUT_DIRECT	( 0x3690, 0x85c )
#endif


#endif //_SX16IOCTL_H

